#!/usr/bin/env python

import sys
import subprocess
import logging
import pymysql.cursors
import re
import sqlite3
import config_parser

BIOS_FIELDS = ['HARDWARE_ID', 'SMANUFACTURER', 'BMANUFACTURER', 'BVERSION', 'SSN', 'SMODEL']
CONTROLLERS_FIELDS = ['HARDWARE_ID','NAME', 'MANUFACTURER', 'CAPTION', 'TYPE']
DEVICES_FIELDS = ['HARDWARE_ID','NAME']
DRIVES_FIELDS = ['HARDWARE_ID','FILESYSTEM', 'VOLUMN', 'TYPE']
INPUTS_FIELDS = ['HARDWARE_ID','CAPTION', 'DESCRIPTION']
MEMORIES_FIELDS = ['HARDWARE_ID','CAPTION', 'DESCRIPTION']
MODEMS_FIELDS = ['HARDWARE_ID','NAME', 'DESCRIPTION', 'MODEL']
MONITORS_FIELDS = ['HARDWARE_ID','MANUFACTURER', 'CAPTION', 'DESCRIPTION', 'TYPE']
NETWORKS_FIELDS = ['HARDWARE_ID','MACADDR', 'DESCRIPTION']
PORTS_FIELDS = ['HARDWARE_ID','CAPTION', 'DESCRIPTION', 'TYPE']
PRINTERS_FIELDS = ['HARDWARE_ID','NAME', 'DRIVER', 'PORT']
SLOTS_FIELDS = ['HARDWARE_ID','NAME', 'DESCRIPTION']
SOUNDS_FIELDS = ['HARDWARE_ID','MANUFACTURER', 'NAME', 'DESCRIPTION']
STORAGES_FIELDS= ['HARDWARE_ID','MANUFACTURER', 'NAME', 'MODEL', 'DESCRIPTION']
VIDEOS_FIELDS = ['HARDWARE_ID','NAME', 'CHIPSET']


class Filelogger():

    def __init__(self,name,formatter,file_path,file_mode):
        self.name = name
        self.formatter = formatter
        self.file_path = file_path
        self.file_mode = file_mode

    def send_log(self,log_level,message):
        logging.basicConfig(format=self.formatter,filename=self.file_path,filemode=self.file_mode,level=logging.DEBUG)
        if log_level == "debug":
            logging.debug(message)
        elif log_level == "info":
            logging.info(message)
        elif log_level == "critical":
            logging.critical(message)
        elif log_level == "warning":
            logging.warning(message)
        elif log_level == "error":
            logging.error(message)
        else:
            pass

logger_error = Filelogger("pyOCSng",'%(asctime)s %(name)s %(levelname)s %(message)s',"/var/log/ahtapot/ahtapot-ocs_wb_alarms_error.log","a")


class DatabaseConnect:

    def __init__(self,host,user,password,database):
        self.logger = Filelogger("pyOCSng",'%(asctime)s %(name)s %(levelname)s %(message)s',"/var/log/ahtapot/ahtapot-ocs_wb_alarms.log","a")
        try:
            self.connection = pymysql.connect(host=host, user=user, password=password, db=database, cursorclass=pymysql.cursors.DictCursor)
        except Exception as e:
            logger_error.send_log("error", "An error occured while connecting to MySQL Server : {} : line : {}".format(str(e),str(sys.exc_info()[2].tb_lineno)))
            exit()

    def query_db(self,query):
        try:
            with self.connection.cursor() as cursor:
                cursor.execute(query)
                return cursor.fetchall()
        except Exception as e:
            logger_error.send_log("error", "An error occured while executing query on MySQL Server : {} : line : {}".format(str(e),str(sys.exc_info()[2].tb_lineno)))
            exit()

    def query_db_hardware(self, table, fields):
        try:
            with self.connection.cursor() as cursor:
                cursor.execute("SELECT " + fields + " FROM " + table)
                return cursor.fetchall()
        except Exception as e:
            logger_error.send_log("error", "An error occured while executing query on MySQL Server : {} : line : {}".format(str(e),str(sys.exc_info()[2].tb_lineno)))
            exit()

    def close_db(self):
        self.connection.close()


class OcsNgOps:

    def __init__(self):
        pass

    def check_values(self, value_list, value_to_compare):
        for value in value_list:
            if re.search(re.escape(value.lower()), value_to_compare.lower()) is None:
                continue
            else:
                return True
        return False

    def read_values(self, file_path):
        values = []
        with open(file_path, "r") as value_file:
            lines = value_file.readlines()
        for line in lines:
            values.append(line.strip())
        return values

    def create_values_softwares(self, sqlitedb, table_name):
        sw_values = []
        connection = sqlite3.connect(sqlitedb)
        cursor = connection.cursor()
        cursor.execute("SELECT software_id FROM " + table_name)
        results = cursor.fetchall()
        for record in results:
            sw_id = (record[0],)
            cursor.execute("SELECT name FROM ocs_db_software WHERE id=?", sw_id)
            result = cursor.fetchone()
            sw_values.append(result[0])
        return sw_values

    def create_values_hardwares(self, sqlitedb, table_name):
        connection = sqlite3.connect(sqlitedb)
        cursor = connection.cursor()
        cursor.execute("SELECT table_name, context FROM " + table_name)
        results = cursor.fetchall()
        return results

    def create_values_hardwares_wl(self, sqlitedb, table_name, table):
        connection = sqlite3.connect(sqlitedb)
        cursor = connection.cursor()
        cursor.execute("SELECT table_name, context FROM " + table_name + " WHERE table_name=\"" + table + "\"")
        results = cursor.fetchall()
        return results

    def concatenate_fields(self, field_list):
        return ",".join(field_list)


def check_softwares():
    try:
        config = config_parser.DmrConfigParser()
        db_host = config.get_db_configs()["host"]
        db_user = config.get_db_configs()["user"]
        db_pass = config.get_db_configs()["password"]
        db_name = config.get_db_configs()["db_name"]
        sqlite_db = config.get_db_configs()["sqlite_db"]
        scan_mode = config.get_db_configs()["scan_mode"]
        db = DatabaseConnect(db_host, db_user, db_pass,db_name)
        softwares = db.query_db("SELECT hardware_id, name, installdate FROM softwares")
        ocs = OcsNgOps()
        if scan_mode == "blacklist":
            values = ocs.create_values_softwares(sqlite_db, 'ocs_db_softwareblacklist')
        else:
            values = ocs.create_values_softwares(sqlite_db, 'ocs_db_softwarewhitelist')
        for software in softwares:
            if scan_mode == "blacklist":
                if ocs.check_values(values, software['name']):
                    machine = db.query_db("SELECT deviceid, name FROM hardware WHERE id=" + str(software['hardware_id']))[0]
                    db.logger.send_log("error", "BLACKLIST: Found a vulnerable software : {} on {}({}) Machine".format(software['name'], machine['name'], machine['deviceid']))
            else:
                if ocs.check_values(values, software['name']) is False:
                    machine = db.query_db("SELECT deviceid, name FROM hardware WHERE id=" + str(software['hardware_id']))[0]
                    db.logger.send_log("error", "WHITELIST: Found a vulnerable software : {} on {}({}) Machine".format(software['name'], machine['name'], machine['deviceid']))
    except Exception as e:
        logger_error.send_log("error", "An error occured while checking softwares : {} : line : {}".format(str(e),str(sys.exc_info()[2].tb_lineno)))


def check_and_log_hardware(db, table_list, blacklist, table_name):
    config = config_parser.DmrConfigParser()
    scan_mode = config.get_db_configs()["scan_mode"]
    try:
        if scan_mode == "blacklist":
            for hw in table_list:
                for bl in blacklist:
                    black_hw = True
                    for field, f_value in hw.iteritems():
                        if field != "hardware_id":
                            if re.search(re.escape(unicode(f_value)), bl) is None:
                                black_hw = False
                                break
                            else:
                                continue
                    if black_hw is True:
                        machine = db.query_db("SELECT deviceid, name FROM hardware WHERE id=" + str(hw[u'HARDWARE_ID']))[0]
                        db.logger.send_log("error", "BLACKLIST: Found a vulnerable "+ table_name +" hardware on {}({}) Machine".format(machine['name'], machine['deviceid']))
        else:
            for hw in table_list:
                for wl in blacklist:
                    white_hw = True
                    for field, f_value in hw.iteritems():
                        if field != "hardware_id":
                            if re.search(re.escape(unicode(f_value)), wl) is None:
                                white_hw = False
                                break
                            else:
                                continue
                    if white_hw is True:
                        break
                if white_hw is False:
                    machine = db.query_db("SELECT deviceid, name FROM hardware WHERE id=" + str(hw[u'HARDWARE_ID']))[0]
                    db.logger.send_log("error", "WHITELIST: Found a vulnerable "+ table_name +" hardware on {}({}) Machine".format(machine['name'], machine['deviceid']))

    except Exception as e:
        logger_error.send_log("error", "An error occured while checking Hardware and logging : {} : line : {}".format(str(e),str(sys.exc_info()[2].tb_lineno)))


def check_hardwares():
    try:
        config = config_parser.DmrConfigParser()
        db_host = config.get_db_configs()["host"]
        db_user = config.get_db_configs()["user"]
        db_pass = config.get_db_configs()["password"]
        db_name = config.get_db_configs()["db_name"]
        sqlite_db = config.get_db_configs()["sqlite_db"]
        scan_mode = config.get_db_configs()["scan_mode"]
        db = DatabaseConnect(db_host, db_user, db_pass,db_name)
        ocs = OcsNgOps()
        if scan_mode == "blacklist":
            hw_blacklist = ocs.create_values_hardwares(sqlite_db, 'ocs_db_hardwareblacklist')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                bios_results = db.query_db_hardware('bios', ','.join(BIOS_FIELDS))
                check_and_log_hardware(db, bios_results, hw_blacklist_context, 'bios')
                controllers_results = db.query_db_hardware('controllers',','.join(CONTROLLERS_FIELDS ))
                check_and_log_hardware(db, controllers_results, hw_blacklist_context, 'controllers')
                drives_results = db.query_db_hardware('drives',','.join(DRIVES_FIELDS ))
                check_and_log_hardware(db, drives_results, hw_blacklist_context, 'drives')
                inputs_results = db.query_db_hardware('inputs',','.join(INPUTS_FIELDS ))
                check_and_log_hardware(db, inputs_results, hw_blacklist_context, 'inputs')
                memories_results = db.query_db_hardware('memories',','.join(MEMORIES_FIELDS ))
                check_and_log_hardware(db, memories_results, hw_blacklist_context, 'memories')
                modems_results = db.query_db_hardware('modems',','.join(MODEMS_FIELDS ))
                check_and_log_hardware(db, modems_results, hw_blacklist_context, 'modems')
                monitors_results = db.query_db_hardware('monitors',','.join(MONITORS_FIELDS ))
                check_and_log_hardware(db, monitors_results, hw_blacklist_context, 'monitors')
                networks_results = db.query_db_hardware('networks',','.join(NETWORKS_FIELDS ))
                check_and_log_hardware(db, networks_results, hw_blacklist_context, 'networks')
                ports_results = db.query_db_hardware('ports',','.join(PORTS_FIELDS ))
                check_and_log_hardware(db, ports_results, hw_blacklist_context, 'ports')
                printers_results = db.query_db_hardware('printers',','.join(PRINTERS_FIELDS ))
                check_and_log_hardware(db, printers_results, hw_blacklist_context, 'printers')
                storages_results = db.query_db_hardware('storages',','.join(STORAGES_FIELDS))
                check_and_log_hardware(db, storages_results, hw_blacklist_context, 'storages')
                videos_results = db.query_db_hardware('videos',','.join(VIDEOS_FIELDS ))
                check_and_log_hardware(db, videos_results, hw_blacklist_context, 'videos')
        else:
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','bios')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                bios_results = db.query_db_hardware('bios', ','.join(BIOS_FIELDS))
                check_and_log_hardware(db, bios_results, hw_blacklist_context, 'bios')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','controllers')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                controllers_results = db.query_db_hardware('controllers',','.join(CONTROLLERS_FIELDS ))
                check_and_log_hardware(db, controllers_results, hw_blacklist_context, 'controllers')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','drives')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                drives_results = db.query_db_hardware('drives',','.join(DRIVES_FIELDS ))
                check_and_log_hardware(db, drives_results, hw_blacklist_context, 'drives')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','inputs')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                inputs_results = db.query_db_hardware('inputs',','.join(INPUTS_FIELDS ))
                check_and_log_hardware(db, inputs_results, hw_blacklist_context, 'inputs')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','memories')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                memories_results = db.query_db_hardware('memories',','.join(MEMORIES_FIELDS ))
                check_and_log_hardware(db, memories_results, hw_blacklist_context, 'memories')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','modems')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                modems_results = db.query_db_hardware('modems',','.join(MODEMS_FIELDS ))
                check_and_log_hardware(db, modems_results, hw_blacklist_context, 'modems')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','monitors')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                monitors_results = db.query_db_hardware('monitors',','.join(MONITORS_FIELDS ))
                check_and_log_hardware(db, monitors_results, hw_blacklist_context, 'monitors')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','networks')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                networks_results = db.query_db_hardware('networks',','.join(NETWORKS_FIELDS ))
                check_and_log_hardware(db, networks_results, hw_blacklist_context, 'networks')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','ports')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                ports_results = db.query_db_hardware('ports',','.join(PORTS_FIELDS ))
                check_and_log_hardware(db, ports_results, hw_blacklist_context, 'ports')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','printers')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                printers_results = db.query_db_hardware('printers',','.join(PRINTERS_FIELDS ))
                check_and_log_hardware(db, printers_results, hw_blacklist_context, 'printers')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','storages')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                storages_results = db.query_db_hardware('storages',','.join(STORAGES_FIELDS))
                check_and_log_hardware(db, storages_results, hw_blacklist_context, 'storages')
            hw_blacklist = ocs.create_values_hardwares_wl(sqlite_db, 'ocs_db_hardwarewhitelist','videos')
            if not len(hw_blacklist) == 0:
                hw_blacklist_context = []
                for hw in hw_blacklist:
                    hw_blacklist_context.append(hw[1])
                videos_results = db.query_db_hardware('videos',','.join(VIDEOS_FIELDS ))
                check_and_log_hardware(db, videos_results, hw_blacklist_context, 'videos')

    except Exception as e:
        logger_error.send_log("error", "An error occured while checking hardware blacklist : {} : line : {}".format(str(e),str(sys.exc_info()[2].tb_lineno)))
if __name__ == "__main__":
    check_softwares()
    check_hardwares()
